//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/SampleView/RealspaceWidget.h
//! @brief     Defines class RealspaceWidget
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_SAMPLEVIEW_REALSPACEWIDGET_H
#define BORNAGAIN_GUI_VIEW_SAMPLEVIEW_REALSPACEWIDGET_H

#include "Img3D/Type/SceneGeometry.h"
#include <QWidget>
#include <memory>

class CautionSign;
class Item3D;
class SampleItem;

namespace Img3D {
class Canvas;
class Model;
} // namespace Img3D

//! Provides 3D object generation for Realspace presentation.
class RealspaceWidget : public QWidget {
    Q_OBJECT
public:
    RealspaceWidget(QWidget* parent = nullptr);
    ~RealspaceWidget() override;

    void setDisplayedItem(SampleItem* containingSample, Item3D* item);
    void clearDisplay() { setDisplayedItem(nullptr, nullptr); }
    void defaultView();
    void sideView();
    void topView();
    void changeLayerSize(double layerSizeChangeScale);
    void savePicture();
    void resetScene();
    void updateScene();
    Item3D* displayedItem() { return m_displayedItem; }

protected:
    void showEvent(QShowEvent*) override;

private:
    void savePicture(const QPixmap& pixmap);

    Img3D::Canvas* m_canvas;
    std::unique_ptr<Img3D::Model> m_realspaceModel;
    SceneGeometry m_sceneGeometry;
    CautionSign* m_cautionSign;
    Item3D* m_displayedItem;
    SampleItem* m_containingSample;
    bool m_firstView;
};

#endif // BORNAGAIN_GUI_VIEW_SAMPLEVIEW_REALSPACEWIDGET_H
